from urllib.parse import parse_qs, urlparse

import requests

NETWORK_URL_SCHEMES = {
    "http",
    "https",
    "irc",
    "ircs",
    "ssh",
}

HANDLE_SCHEMES = {
    "mailto",
    "xmpp",
}

PHONE_SCHEMES = {
    "sms",
    "tel",
}


def ensure_url_has_scheme(url):
    if urlparse(url).scheme:
        return url

    if not url.startswith(("//",)):
        return f"http://{url}"
    return url


def is_valid_url(url):
    try:
        result = urlparse(url)
        scheme = result.scheme

        if scheme in NETWORK_URL_SCHEMES:
            return bool(result.netloc)

        if scheme in HANDLE_SCHEMES:
            return bool(result.path and "@" in result.path)

        if scheme in PHONE_SCHEMES:
            return bool(result.path)

        if scheme == "magnet":
            query = parse_qs(result.query)
            return bool(query.get("xt"))

        return False
    except ValueError:
        return False


def shorten_url(url):
    url = url.strip()
    url = ensure_url_has_scheme(url)

    if not is_valid_url(url):
        raise ValueError(f"'{url}' is not a valid supported URL.")

    response = requests.post(
        "https://s.lain.la",
        data={"url": url},
        headers={"Content-Type": "application/x-www-form-urlencoded"},
        timeout=10,
    )
    response.raise_for_status()
    result = response.text.strip()
    if not result:
        raise ValueError("Server returned an empty response.")

    return result



import argparse
import sys
import webbrowser

import requests

from .shorten import shorten_url
from .version import __version__


def main():
    parser = argparse.ArgumentParser(
        description="Shorten URLs using lain.la API",
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        epilog="Example: %(prog)s https://kuroneko.dev/",
    )
    parser.add_argument("-v", "--version", action="version", version=__version__)
    parser.add_argument(
        "-o",
        "--open",
        action="store_true",
        help="open each generated short URL in your default browser",
    )
    parser.add_argument("urls", nargs="+", help="URL(s) to be shortened")

    args = parser.parse_args()

    shortened_urls = []
    has_error = False

    for url_full in args.urls:
        try:
            url_short = shorten_url(url_full)
            if url_short.startswith("http"):
                print(f"Shortened URL: {url_short}")
                shortened_urls.append(url_short)
            else:
                print(
                    f"Error: API returned invalid format for {url_full}: {url_short}",
                    file=sys.stderr,
                )
                has_error = True
        except ValueError as e:
            print(f"Value Error: {e}", file=sys.stderr)
            has_error = True
            continue
        except requests.RequestException as e:
            print(f"Network error: {e}", file=sys.stderr)
            has_error = True
            continue

    if shortened_urls:
        all_urls = "\n".join(shortened_urls)
        try:
            import pyperclip

            pyperclip.copy(all_urls)
            print("\nURL(s) copied to clipboard", file=sys.stderr)
        except Exception:
            pass

        if args.open:
            for url_short in shortened_urls:
                webbrowser.open_new_tab(url_short)

    if has_error:
        sys.exit(1)
